/*=========================================================================*/
/* SEED reader     |           time_span_out             |    subprocedure */
/*=========================================================================*/
/*
	Name:		time_span_out
	Purpose:	to select seismic data according to user time span input.
	Usage:		void time_span_out ();
				time_span_out ();
	Input:	none - all data taken from globally available variables.
	Output:		none
	Externals: data_hdr
			start_time_count
			start_time_point
			end_time_count
			end_time_point
	Warnings:	
	Errors:		none
	Called by:	process_data
	Calls to:	none
	Algorithm:
	Problems:	none known
	References:	Halbert, S. E., R. Buland, and C. R. Hutt (1988).  Standard for
	the Exchange of Earthquake Data (SEED), Version V2.0,
	February 25, 1988.  United States Geological Survey,
	Albuquerque Seismological Laboratory, Building 10002,
	Kirtland Air Force Base East, Albuquerque, New Mexico
	87115.  82 pp.
	O'Neill, D. (1987).  IRIS Interim Data Distribution Format
	(SAC ASCII), Version 1.0 (12 November 1987).  Incorporated
	Research Institutions for Seismology, 1616 North Fort Myer
	Drive, Suite 1440, Arlington, Virginia 22209.  11 pp.
	Tull, J. (1987).  SAC User's Manual, Version 10.2, October 7,
	1987.  Lawrence Livermore National Laboratory, L-205,
	Livermore, California 94550.  ??? pp.
	Language:	C, hopefully ANSI standard
	Author:		Allen Nance
	Revisions:
		07/94    CL - took out the breaks so that if the user
				specified time slices they would actually
				get processed.

		01/25/96 CL - cast the offset * TIME_PRECISION to a 
				float to avoid overflow errors.

*/

#include "rdseed.h"								/* SEED tables and structures */

#include "utilities.h"

#include <string.h>

#define SUBSTR(str,xch,ch) { char *c; while ((c = strchr(str, xch))) *c = ch; }

#define TIME_PRECISION 10000

// defined in rdseed.c
extern int blk_71_do_once;

extern int read_summary_flag;   /* defined and set in main() */

struct twindow			/* used if from summary file */
{
        char time_start[23];
        char time_end[23];
	int already_used;
        char chn_list[512];

        struct twindow *next;
};

struct twindow *get_twin_from_summary();
struct twindow *scan_summary_file();
float summary_file_get_version();
int getNumEvents();

/* ------------------------------------------------------------------------ */
char *summary_file_get_event();


/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */

void process_type71()

{
	char b71[255];

	int nummags;

	char blklen[5];

	char otime[32];
	char lat[20];
	char lon[21];
	char depth[20];
	char nmags[3];
	char mags[10];
	char mtype[10];
	char **parts;
	int i, n;

	char *ev;

	float weed_version;

	struct time ev_time;

	if (!read_summary_flag)
		return;

	if (getNumEvents() != 1)
		return;

	weed_version = summary_file_get_version();

       	ev = summary_file_get_event();

	// init some fields
	memset(otime, '~', 22);
	memset(mtype, '~', 10);

	memset(b71, ' ', sizeof(b71));

	strcpy(b71, "");

	strcat(b71, "071");
	strcat(b71, "    ");		// figure out later

	// old weed (version < 3.0 has seismic code and region added

	// only support summary file version 4.0 and higher

	if (weed_version < 4.0)
	{
		save_myfprintf(stderr, "Error - rdseed only supports summary file versions 4.0 and higher!\n");
		save_myfprintf(stderr, "Unable to parse event fields from the summary file\n");
		save_myfprintf(stderr, "Event: %s\n", summary_file_get_event());
		return;
	}

	// an event line looks like:
	//	2012-05-03 13:38:35.2700,2.566,94.543,29.6,MB,4.9

	n = split(ev, &parts, ',');

	SUBSTR(parts[0], '-', '/');

	timecvt(&ev_time, parts[0]);
			
	sprintf(otime, "%d,%d,%d.%d.%d.%d~", ev_time.year, ev_time.day, ev_time.hour, ev_time.minute, ev_time.second, ev_time.fracsec);

	strcat(b71, otime);

	strcat(b71, "  ");		// source, not used

	sprintf(lat, "%-10s", parts[1]);

	strcat(b71, lat);

	sprintf(lon, "%-11s", parts[2]);

	strcat(b71, lon);

	sprintf(depth, "%-7s", parts[3]);

	strcat(b71, depth);
		
	// peel off mag type and mag, start at field 4

	if (n < 6)		// type and mag
	{
		save_myfprintf(stderr, "Error - No mag type and mag in event from summary file.\n");
		save_myfprintf(stderr, "Unable to parse event fields from the summary file\n");
		save_myfprintf(stderr, "Event: %s\n", summary_file_get_event());
	}
		
	nummags = (n - 4)/2;

	sprintf(nmags, "%-2d", nummags);

	strcat(b71, nmags);
	
	for (i = 4; i < nummags + 4; i += 2)		// by two because both man and type are processed
	{
		// an event line looks like:
              	//      2012-05-03 13:38:35.2700,2.566,94.543,29.6,MB,4.9

		sprintf(mtype, "%s~", parts[i]);
		sprintf(mags, "%-5s", parts[i + 1]);

		strcat(b71, mags);

		strcat(b71, mtype);

		strcat(b71, "   ");		// source not available

	}

	fuse(&parts, n);

	sprintf(blklen, "%-4d", (int)strlen(b71));

	memcpy(&b71[3], blklen, 4);

	update_type71(b71);

}

/* ------------------------------------------------------------------------ */
void time_span_out ()
{
	int i;										/* counter */
	struct data_hdr temp_hdr;	/* data header */
	struct time end_time;		/* block ending time */
	int offset;			/* local computations */
	double duration;

	struct type52 *save_channel_ptr = current_channel;

	struct time endtime;

/*  
 * if output is intended for a seed volume, update the blockette info 74
 * only...
 *
 */
/*                +=======================================+                */
/*================|          Look for Time Spans          |================*/
/*                +=======================================+                */
	if (read_summary_flag)
	{

		duration = 
		   ((double)(data_hdr->nsamples)*TIME_PRECISION)/data_hdr->sample_rate;

		endtime = timeadd_double(data_hdr->time, duration);

		set_event(&(data_hdr->time), &endtime);

	}

	if (Seed_flag)
	{
		if (blk_71_do_once)
		{
			process_type71();	
			blk_71_do_once = FALSE;
		}	

		update_type74(data_hdr);
		data_hdr->nsamples = 0;

		return;
	}
	
	/* if user didn't enter any time spans, output all data */
	if (start_time_count == 0)
	{
		output_data(data_hdr, 0);
		data_hdr->nsamples = 0;

		return;
	}
	/* look through time span list as entered by the user */
	for (i=0;i<start_time_count;i++)
	{
		current_channel = save_channel_ptr;

		/* copy current data header */
		temp_hdr = *data_hdr;

		/* is span start equal to start of data? */
		if (timecmp (start_time_point[i],data_hdr->time) == 0)
		{
			/* is there a corresponding end time? */
			if (i < end_time_count)
			{
				/* use current data start time */
				/* calc number of samples */

				duration = timedif(temp_hdr.time,end_time_point[i]);

				/* took this out of timedif */
				if (duration < 0)
					duration = 0;

				temp_hdr.nsamples = (int) (((duration*temp_hdr.sample_rate)/TIME_PRECISION)+.999);

				/* make sure it's not more than available data */
				if (temp_hdr.nsamples > data_hdr->nsamples) temp_hdr.nsamples = data_hdr->nsamples;

				output_data(&temp_hdr, 0);

			}
			else 
			{
				output_data(data_hdr, 0);
			}
		}
		/* is span start less than start of data? */
		else if (timecmp (start_time_point[i],data_hdr->time) < 0)
		{
			/* is there a corresponding end time? */
			if (i < end_time_count)
			{
				if (timecmp (end_time_point[i],data_hdr->time) > 0)
				{
					/* use current data start time */
					/* calc number of samples */

					duration = timedif(temp_hdr.time,end_time_point[i]);

                                	/* took this out of timedif */
                                	if (duration < 0)
                                        	duration = 0;

					temp_hdr.nsamples = (int) (((duration*temp_hdr.sample_rate)/TIME_PRECISION)+.999);

					/* make sure it's not more than available data */
					if (end_time_point[i].year == 9999) temp_hdr.nsamples = data_hdr->nsamples;

					if (temp_hdr.nsamples > data_hdr->nsamples) temp_hdr.nsamples = data_hdr->nsamples;
					output_data(&temp_hdr, 0);
				}
			}
			else
			{
				output_data(data_hdr, 0);
			}
	
		}
		/* span start must be greater than start of data */
		else
		{
			/* figure out data ending time */
			if (data_hdr->sample_rate != 0)
				duration = (((double)data_hdr->nsamples * TIME_PRECISION) / data_hdr->sample_rate);

			else 
				duration = 0;

			end_time = timeadd_double (data_hdr->time, duration);

			/* is span start before end of data? */
			if (timecmp(start_time_point[i],end_time) < 0)
			{
				/* calc new start time and sample offset*/
				duration = timedif(temp_hdr.time,start_time_point[i]);

				/* took this out of timedif */
				if (duration < 0)
					duration = 0;

				offset = (int) ((duration * data_hdr->sample_rate)/TIME_PRECISION);

				/* changed to float to avoid overflow - CL */
				duration = 
					((float) offset * TIME_PRECISION) / 
						data_hdr->sample_rate;

				temp_hdr.time = timeadd_double (data_hdr->time, duration);
				/* calc number of samples */
				if (i < end_time_count)
				{
					if (end_time_point[i].year != 9999)
						duration = timedif(temp_hdr.time,end_time_point[i]);
					else
						duration = timedif(temp_hdr.time,end_time);

				}
				else 
					duration = timedif(temp_hdr.time,end_time);

				/* took this out of timedif */
				if (duration < 0)
					duration = 0;

				temp_hdr.nsamples = (int)(((duration*temp_hdr.sample_rate)/TIME_PRECISION)+.999);

				/* make sure it's not more than available data */
				if ((temp_hdr.nsamples+offset) > data_hdr->nsamples)
					temp_hdr.nsamples = data_hdr->nsamples - offset;

				output_data(&temp_hdr, offset);

			}
		}
	}

	data_hdr->nsamples = 0;

	if (read_summary_flag)
		/* will avoid unnecessary free()s as these weren't malloced */
                start_time_count = end_time_count = 0;
		
	
/*                +=======================================+                */
/*================|                clean-up               |================*/
/*                +=======================================+                */

}
